@@ -18,10 +18,8 @@ import logging
 import math
 import os
 import re
-import requests
 import shlex
 import shutil
-import six
 import stat
 import tempfile
 import time
@@ -32,11 +30,10 @@ from oslo_serialization import base64
 from oslo_utils import excutils
 from oslo_utils import imageutils
 from oslo_utils import units
+import requests
+import six
 
 from ironic_lib.common.i18n import _
-from ironic_lib.common.i18n import _LE
-from ironic_lib.common.i18n import _LI
-from ironic_lib.common.i18n import _LW
 from ironic_lib import disk_partitioner
 from ironic_lib import exception
 from ironic_lib import utils
@@ -97,9 +94,9 @@ def list_partitions(device):
     for line in lines:
         match = _PARTED_PRINT_RE.match(line)
         if match is None:
-            LOG.warning(_LW("Partition information from parted for device "
-                            "%(device)s does not match "
-                            "expected format: %(line)s"),
+            LOG.warning("Partition information from parted for device "
+                        "%(device)s does not match "
+                        "expected format: %(line)s",
                         dict(device=device, line=line))
             continue
         # Cast int fields to ints (some are floats and we round them down)
@@ -162,6 +159,11 @@ def is_iscsi_device(dev, node_uuid):
     return iscsi_id in dev
 
 
+def is_nvme_device(dev):
+    """check whether the device path belongs to an NVMe drive. """
+    return "/dev/nvme" in dev
+
+
 def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
                     configdrive_mb, node_uuid, commit=True,
                     boot_option="netboot", boot_mode="bios",
@@ -201,6 +203,8 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     # the device partitions as /dev/sda1 and not /dev/sda-part1.
     if is_iscsi_device(dev, node_uuid):
         part_template = dev + '-part%d'
+    elif is_nvme_device(dev):
+        part_template = dev + 'p%d'
     else:
         part_template = dev + '%d'
     part_dict = {}
@@ -370,8 +374,11 @@ def destroy_disk_metadata(dev, node_uuid):
                               run_as_root=True,
                               use_standard_locale=True)
 
-    LOG.info(_LI("Disk metadata on %(dev)s successfully destroyed for node "
-                 "%(node)s"), {'dev': dev, 'node': node_uuid})
+    utils.execute('sgdisk', '-Z', dev, run_as_root=True,
+                  use_standard_locale=True)
+
+    LOG.info("Disk metadata on %(dev)s successfully destroyed for node "
+             "%(node)s", {'dev': dev, 'node': node_uuid})
 
 
 def _get_configdrive(configdrive, node_uuid, tempdir=None):
@@ -489,8 +496,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
                                     boot_option=boot_option,
                                     boot_mode=boot_mode,
                                     disk_label=disk_label)
-        LOG.info(_LI("Successfully completed the disk device"
-                     " %(dev)s partitioning for node %(node)s"),
+        LOG.info("Successfully completed the disk device"
+                 " %(dev)s partitioning for node %(node)s",
                  {'dev': dev, "node": node_uuid})
 
         ephemeral_part = part_dict.get('ephemeral')
@@ -522,8 +529,8 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
         if configdrive_part:
             # Copy the configdrive content to the configdrive partition
             dd(configdrive_file, configdrive_part)
-            LOG.info(_LI("Configdrive for node %(node)s successfully copied "
-                         "onto partition %(partition)s"),
+            LOG.info("Configdrive for node %(node)s successfully copied "
+                     "onto partition %(partition)s",
                      {'node': node_uuid, 'partition': configdrive_part})
 
     finally:
@@ -533,20 +540,20 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
             utils.unlink_without_raise(configdrive_file)
 
     populate_image(image_path, root_part)
-    LOG.info(_LI("Image for %(node)s successfully populated"),
+    LOG.info("Image for %(node)s successfully populated",
              {'node': node_uuid})
 
     if swap_part:
         utils.mkfs(fs='swap', path=swap_part, label='swap1')
-        LOG.info(_LI("Swap partition %(swap)s successfully formatted "
-                     "for node %(node)s"),
+        LOG.info("Swap partition %(swap)s successfully formatted "
+                 "for node %(node)s",
                  {'swap': swap_part, 'node': node_uuid})
 
     if ephemeral_part and not preserve_ephemeral:
         utils.mkfs(fs=ephemeral_format, path=ephemeral_part,
                    label="ephemeral0")
-        LOG.info(_LI("Ephemeral partition %(ephemeral)s successfully "
-                     "formatted for node %(node)s"),
+        LOG.info("Ephemeral partition %(ephemeral)s successfully "
+                 "formatted for node %(node)s",
                  {'ephemeral': ephemeral_part, 'node': node_uuid})
 
     uuids_to_return = {
@@ -561,7 +568,7 @@ def work_on_disk(dev, root_mb, swap_mb, ephemeral_mb, ephemeral_format,
 
     except processutils.ProcessExecutionError:
         with excutils.save_and_reraise_exception():
-            LOG.error(_LE("Failed to detect %s"), part)
+            LOG.error("Failed to detect %s", part)
 
     return uuids_to_return
 
@@ -581,9 +588,10 @@ def _is_disk_larger_than_max_size(device, node_uuid):
     :returns: True if total disk size exceeds 2TB. Returns False otherwise.
     """
     try:
-        disksize_bytes = utils.execute('blockdev', '--getsize64', device,
-                                       use_standard_locale=True,
-                                       run_as_root=True)
+        disksize_bytes, err = utils.execute('blockdev', '--getsize64',
+                                            device,
+                                            use_standard_locale=True,
+                                            run_as_root=True)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
         msg = (_('Failed to get size of disk %(disk)s for node %(node)s. '
@@ -592,7 +600,7 @@ def _is_disk_larger_than_max_size(device, node_uuid):
         LOG.error(msg)
         raise exception.InstanceDeployFailure(msg)
 
-    disksize_mb = int(disksize_bytes[0].strip()) // 1024 // 1024
+    disksize_mb = int(disksize_bytes.strip()) // 1024 // 1024
 
     return disksize_mb > MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR
 
@@ -767,9 +775,9 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 endlimit = '-0'
                 if _is_disk_larger_than_max_size(device, node_uuid):
                     # Need to create a small partition at 2TB limit
-                    LOG.warning(_LW("Disk size is larger than 2TB for "
-                                    "node %(node)s. Creating config drive "
-                                    "at the end of the disk %(disk)s."),
+                    LOG.warning("Disk size is larger than 2TB for "
+                                "node %(node)s. Creating config drive "
+                                "at the end of the disk %(disk)s.",
                                 {'node': node_uuid, 'disk': device})
                     startlimit = (MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR -
                                   MAX_CONFIG_DRIVE_SIZE_MB - 1)
@@ -789,20 +797,28 @@ def create_config_drive_partition(node_uuid, device, configdrive):
 
             if is_iscsi_device(device, node_uuid):
                 config_drive_part = '%s-part%s' % (device, new_part.pop())
+            elif is_nvme_device(device):
+                config_drive_part = '%sp%s' % (device, new_part.pop())
             else:
                 config_drive_part = '%s%s' % (device, new_part.pop())
 
-            # NOTE(vdrok): the partition was created successfully, let's wait
-            # for it to appear in /dev.
+            LOG.debug('Waiting until udev event queue is empty')
+            utils.execute('udevadm', 'settle')
+
+            # NOTE(vsaienko): check that devise actually exists,
+            # it is not handled by udevadm when using ISCSI, for more info see:
+            # https://bugs.launchpad.net/ironic/+bug/1673731
+            # Do not use 'udevadm settle --exit-if-exist' here
             LOG.debug('Waiting for the config drive partition %(part)s '
                       'on node %(node)s to be ready for writing.',
                       {'part': config_drive_part, 'node': node_uuid})
-            utils.execute('udevadm', 'settle',
-                          '--exit-if-exists=%s' % config_drive_part)
+            utils.execute('test', '-e', config_drive_part,
+                          check_exit_code=[0], attempts=15,
+                          delay_on_retry=True)
 
         dd(confdrive_file, config_drive_part)
-        LOG.info(_LI("Configdrive for node %(node)s successfully "
-                     "copied onto partition %(part)s"),
+        LOG.info("Configdrive for node %(node)s successfully "
+                 "copied onto partition %(part)s",
                  {'node': node_uuid, 'part': config_drive_part})
 
     except (processutils.UnknownArgumentError,
