@@ -14,30 +14,30 @@
 #    under the License.
 
 import gzip
-import mock
 import os
 import shutil
 import stat
 import tempfile
 
+import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_serialization import base64
 from oslo_service import loopingcall
 from oslo_utils import imageutils
-from oslotest import base as test_base
 import requests
 
 from ironic_lib import disk_partitioner
 from ironic_lib import disk_utils
 from ironic_lib import exception
+from ironic_lib.tests import base
 from ironic_lib import utils
 
 CONF = cfg.CONF
 
 
 @mock.patch.object(utils, 'execute', autospec=True)
-class ListPartitionsTestCase(test_base.BaseTestCase):
+class ListPartitionsTestCase(base.IronicLibTestCase):
 
     def test_correct(self, execute_mock):
         output = """
@@ -72,7 +72,7 @@ BYT;
 
 
 @mock.patch.object(disk_partitioner.DiskPartitioner, 'commit', lambda _: None)
-class WorkOnDiskTestCase(test_base.BaseTestCase):
+class WorkOnDiskTestCase(base.IronicLibTestCase):
 
     def setUp(self):
         super(WorkOnDiskTestCase, self).setUp()
@@ -294,7 +294,7 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
 
 
 @mock.patch.object(utils, 'execute', autospec=True)
-class MakePartitionsTestCase(test_base.BaseTestCase):
+class MakePartitionsTestCase(base.IronicLibTestCase):
 
     def setUp(self):
         super(MakePartitionsTestCase, self).setUp()
@@ -421,6 +421,29 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
         mock_exc.assert_has_calls([parted_call])
         self.assertEqual(expected_result, result)
 
+    def test_make_partitions_with_nvme_device(self, mock_exc):
+        self.ephemeral_mb = 2048
+        expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
+                           'mkpart', 'primary', 'linux-swap', '2049', '2561',
+                           'mkpart', 'primary', '', '2561', '3585']
+        self.dev = '/dev/nvmefake-9'
+        ep = '/dev/nvmefake-9p1'
+        swap = '/dev/nvmefake-9p2'
+        root = '/dev/nvmefake-9p3'
+        expected_result = {'ephemeral': ep,
+                           'swap': swap,
+                           'root': root}
+        cmd = self._get_parted_cmd(self.dev) + expected_mkpart
+        mock_exc.return_value = (None, None)
+        result = disk_utils.make_partitions(
+            self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
+            self.configdrive_mb, self.node_uuid)
+
+        parted_call = mock.call(*cmd, use_standard_locale=True,
+                                run_as_root=True, check_exit_code=[0])
+        mock_exc.assert_has_calls([parted_call])
+        self.assertEqual(expected_result, result)
+
     def test_make_partitions_with_local_device(self, mock_exc):
         self.ephemeral_mb = 2048
         expected_mkpart = ['mkpart', 'primary', '', '1', '2049',
@@ -443,7 +466,7 @@ class MakePartitionsTestCase(test_base.BaseTestCase):
 
 
 @mock.patch.object(utils, 'execute', autospec=True)
-class DestroyMetaDataTestCase(test_base.BaseTestCase):
+class DestroyMetaDataTestCase(base.IronicLibTestCase):
 
     def setUp(self):
         super(DestroyMetaDataTestCase, self).setUp()
@@ -452,6 +475,9 @@ class DestroyMetaDataTestCase(test_base.BaseTestCase):
 
     def test_destroy_disk_metadata(self, mock_exec):
         expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True)]
         disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
@@ -469,9 +495,25 @@ class DestroyMetaDataTestCase(test_base.BaseTestCase):
                           self.node_uuid)
         mock_exec.assert_has_calls(expected_call)
 
+    def test_destroy_disk_metadata_sgdisk_fail(self, mock_exec):
+        expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('sgdisk', '-Z', 'fake-dev',
+                                    run_as_root=True,
+                                    use_standard_locale=True)]
+        mock_exec.side_effect = [(None, None),
+                                 processutils.ProcessExecutionError()]
+        self.assertRaises(processutils.ProcessExecutionError,
+                          disk_utils.destroy_disk_metadata,
+                          self.dev,
+                          self.node_uuid)
+        mock_exec.assert_has_calls(expected_calls)
+
     def test_destroy_disk_metadata_wipefs_not_support_force(self, mock_exec):
         mock_exec.side_effect = iter(
             [processutils.ProcessExecutionError(description='--force'),
+             (None, None),
              (None, None)])
 
         expected_call = [mock.call('wipefs', '--force', '--all', 'fake-dev',
@@ -485,7 +527,7 @@ class DestroyMetaDataTestCase(test_base.BaseTestCase):
 
 
 @mock.patch.object(utils, 'execute', autospec=True)
-class GetDeviceBlockSizeTestCase(test_base.BaseTestCase):
+class GetDeviceBlockSizeTestCase(base.IronicLibTestCase):
 
     def setUp(self):
         super(GetDeviceBlockSizeTestCase, self).setUp()
@@ -503,10 +545,7 @@ class GetDeviceBlockSizeTestCase(test_base.BaseTestCase):
 @mock.patch.object(disk_utils, 'dd', autospec=True)
 @mock.patch.object(disk_utils, 'qemu_img_info', autospec=True)
 @mock.patch.object(disk_utils, 'convert_image', autospec=True)
-class PopulateImageTestCase(test_base.BaseTestCase):
-
-    def setUp(self):
-        super(PopulateImageTestCase, self).setUp()
+class PopulateImageTestCase(base.IronicLibTestCase):
 
     def test_populate_raw_image(self, mock_cg, mock_qinfo, mock_dd):
         type(mock_qinfo.return_value).file_format = mock.PropertyMock(
@@ -537,7 +576,7 @@ def _looping_call_done(*args, **kwargs):
 @mock.patch.object(utils, 'mkfs', lambda fs, path, label=None: None)
 # NOTE(dtantsur): destroy_disk_metadata resets file size, disabling it
 @mock.patch.object(disk_utils, 'destroy_disk_metadata', lambda *_: None)
-class RealFilePartitioningTestCase(test_base.BaseTestCase):
+class RealFilePartitioningTestCase(base.IronicLibTestCase):
     """This test applies some real-world partitioning scenario to a file.
 
     This test covers the whole partitioning, mocking everything not possible
@@ -545,6 +584,9 @@ class RealFilePartitioningTestCase(test_base.BaseTestCase):
     and also conducts integration testing of DiskPartitioner.
     """
 
+    # Allow calls to utils.execute() and related functions
+    block_execute = False
+
     def setUp(self):
         super(RealFilePartitioningTestCase, self).setUp()
         # NOTE(dtantsur): no parted utility on gate-ironic-python26
@@ -608,7 +650,7 @@ class RealFilePartitioningTestCase(test_base.BaseTestCase):
 
 @mock.patch.object(shutil, 'copyfileobj', autospec=True)
 @mock.patch.object(requests, 'get', autospec=True)
-class GetConfigdriveTestCase(test_base.BaseTestCase):
+class GetConfigdriveTestCase(base.IronicLibTestCase):
 
     @mock.patch.object(gzip, 'GzipFile', autospec=True)
     def test_get_configdrive(self, mock_gzip, mock_requests, mock_copy):
@@ -664,7 +706,7 @@ class GetConfigdriveTestCase(test_base.BaseTestCase):
 
 
 @mock.patch('time.sleep', lambda sec: None)
-class OtherFunctionTestCase(test_base.BaseTestCase):
+class OtherFunctionTestCase(base.IronicLibTestCase):
 
     @mock.patch.object(os, 'stat', autospec=True)
     @mock.patch.object(stat, 'S_ISBLK', autospec=True)
@@ -772,7 +814,7 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
 
 
 @mock.patch.object(utils, 'execute', autospec=True)
-class WholeDiskPartitionTestCases(test_base.BaseTestCase):
+class WholeDiskPartitionTestCases(base.IronicLibTestCase):
 
     def setUp(self):
         super(WholeDiskPartitionTestCases, self).setUp()
@@ -947,7 +989,7 @@ or continue with the current setting?
         self.assertEqual(1, mock_log.call_count)
 
 
-class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
+class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
 
     def setUp(self):
         super(WholeDiskConfigDriveTestCases, self).setUp()
@@ -1047,9 +1089,11 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
                       run_as_root=True),
-            mock.call('udevadm', 'settle',
-                      '--exit-if-exists=%s' % expected_part),
+            mock.call('udevadm', 'settle'),
+            mock.call('test', '-e', expected_part, attempts=15,
+                      check_exit_code=[0], delay_on_retry=True)
         ])
+
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_fix_gpt.assert_called_with(self.dev, self.node_uuid)
@@ -1058,7 +1102,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
 
     @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
-    @mock.patch.object(disk_utils.LOG, 'warning')
+    @mock.patch.object(disk_utils.LOG, 'warning', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
@@ -1082,7 +1126,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                    mock_disk_exceeds, mock_dd,
                                    mock_unlink, mock_log, mock_execute,
                                    mock_count, disk_size_exceeds_max=False,
-                                   is_iscsi_device=False):
+                                   is_iscsi_device=False,
+                                   is_nvme_device=False):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1117,6 +1162,9 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
             self.dev = ('/dev/iqn.2008-10.org.openstack:%s.fake' %
                         self.node_uuid)
             expected_part = '%s-part4' % self.dev
+        elif is_nvme_device:
+            self.dev = '/dev/nvmefake'
+            expected_part = '%sp4' % self.dev
         else:
             expected_part = '/dev/fake4'
 
@@ -1137,8 +1185,9 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
-            mock.call('udevadm', 'settle',
-                      '--exit-if-exists=%s' % expected_part),
+            mock.call('udevadm', 'settle'),
+            mock.call('test', '-e', expected_part, attempts=15,
+                      check_exit_code=[0], delay_on_retry=True)
         ])
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
@@ -1150,11 +1199,23 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
 
     def test__create_partition_mbr_disk_under_2TB(self):
         self._test_create_partition_mbr(disk_size_exceeds_max=False,
-                                        is_iscsi_device=True)
+                                        is_iscsi_device=True,
+                                        is_nvme_device=False)
+
+    def test__create_partition_mbr_disk_under_2TB_nvme(self):
+        self._test_create_partition_mbr(disk_size_exceeds_max=False,
+                                        is_iscsi_device=False,
+                                        is_nvme_device=True)
 
     def test__create_partition_mbr_disk_exceeds_2TB(self):
         self._test_create_partition_mbr(disk_size_exceeds_max=True,
-                                        is_iscsi_device=False)
+                                        is_iscsi_device=False,
+                                        is_nvme_device=False)
+
+    def test__create_partition_mbr_disk_exceeds_2TB_nvme(self):
+        self._test_create_partition_mbr(disk_size_exceeds_max=True,
+                                        is_iscsi_device=False,
+                                        is_nvme_device=True)
 
     @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
