@@ -34,8 +34,6 @@ import six
 from six.moves.urllib import parse
 
 from ironic_lib.common.i18n import _
-from ironic_lib.common.i18n import _LE
-from ironic_lib.common.i18n import _LW
 from ironic_lib import exception
 
 utils_opts = [
@@ -134,12 +132,12 @@ def mkfs(fs, path, label=None):
         with excutils.save_and_reraise_exception() as ctx:
             if os.strerror(errno.ENOENT) in e.stderr:
                 ctx.reraise = False
-                LOG.exception(_LE('Failed to make file system. '
-                                  'File system %s is not supported.'), fs)
+                LOG.exception('Failed to make file system. '
+                              'File system %s is not supported.', fs)
                 raise exception.FileSystemNotSupported(fs=fs)
             else:
-                LOG.exception(_LE('Failed to create a file system '
-                                  'in %(path)s. Error: %(error)s'),
+                LOG.exception('Failed to create a file system '
+                              'in %(path)s. Error: %(error)s',
                               {'path': path, 'error': e})
 
 
@@ -150,7 +148,7 @@ def unlink_without_raise(path):
         if e.errno == errno.ENOENT:
             return
         else:
-            LOG.warning(_LW("Failed to unlink %(path)s, error: %(e)s"),
+            LOG.warning("Failed to unlink %(path)s, error: %(e)s",
                         {'path': path, 'e': e})
 
 
@@ -356,10 +354,13 @@ def match_root_device_hints(devices, root_device_hints):
     :returns: The first device to match all the hints or None.
     """
     LOG.debug('Trying to find a device from "%(devs)s" that matches the '
-              'root device hints "%(hints)s"', {'devs': devices,
-                                                'hints': root_device_hints})
+              'root device hints "%(hints)s"',
+              {'devs': ', '.join([d.get('name') for d in devices]),
+               'hints': root_device_hints})
     parsed_hints = parse_root_device_hints(root_device_hints)
     for dev in devices:
+        device_name = dev.get('name')
+
         for hint in parsed_hints:
             hint_type = VALID_ROOT_DEVICE_HINTS[hint]
             device_value = dev.get(hint)
@@ -371,11 +372,23 @@ def match_root_device_hints(devices, root_device_hints):
                                                               hint)
                 except ValueError:
                     LOG.warning(
-                        _LW('The attribute "%(attr)s" of the device "%(dev)s" '
-                            'has an empty value. Skipping device.'),
-                        {'attr': hint, 'dev': dev})
+                        'The attribute "%(attr)s" of the device "%(dev)s" '
+                        'has an empty value. Skipping device.',
+                        {'attr': hint, 'dev': device_name})
                     break
 
+            if hint == 'size':
+                # Since we don't support units yet we expect the size
+                # in GiB for now
+                device_value = device_value / units.Gi
+
+            LOG.debug('Trying to match the root device hint "%(hint)s" '
+                      'with a value of "%(hint_value)s" against the same '
+                      'device\'s (%(dev)s) attribute with a value of '
+                      '"%(dev_value)s"', {'hint': hint, 'dev': device_name,
+                                          'hint_value': hint_value,
+                                          'dev_value': device_value})
+
             # NOTE(lucasagomes): Boolean hints are not supported by
             # specs_matcher.match(), so we need to do the comparison
             # ourselves
@@ -384,26 +397,21 @@ def match_root_device_hints(devices, root_device_hints):
                     device_value = strutils.bool_from_string(device_value,
                                                              strict=True)
                 except ValueError:
-                    LOG.warning(_LW('The attribute "%(attr)s" (with value '
-                                    '"%(value)s") of device "%(dev)s" is not '
-                                    'a valid Boolean. Skipping device.'),
+                    LOG.warning('The attribute "%(attr)s" (with value '
+                                '"%(value)s") of device "%(dev)s" is not '
+                                'a valid Boolean. Skipping device.',
                                 {'attr': hint, 'value': device_value,
-                                 'dev': dev})
+                                 'dev': device_name})
                     break
                 if device_value == hint_value:
                     continue
                 break
 
-            if hint == 'size':
-                # Since we don't support units yet we expect the size
-                # in GiB for now
-                device_value = device_value / units.Gi
-
             if not specs_matcher.match(device_value, hint_value):
                 break
         else:
-            LOG.debug('Device found! The device "%s" matches the root '
-                      'device hints' % dev)
+            LOG.info('Device found! The device "%s" matches the root '
+                     'device hints', device_name)
             return dev
 
-    LOG.debug('No device found that matches the root device hints')
+    LOG.warning('No device found that matches the root device hints')
